@@ -1,5 +1,16 @@
-package PortageXS::Core;
+use strict;
+use warnings;
 
+package PortageXS::Core;
+BEGIN {
+  $PortageXS::Core::AUTHORITY = 'cpan:KENTNL';
+}
+{
+  $PortageXS::Core::VERSION = '0.2.11';
+}
+
+# ABSTRACT: Core behaviour role for C<PortageXS>
+#
 # -----------------------------------------------------------------------------
 #
 # PortageXS::Core
@@ -23,6 +34,7 @@ require Exporter;
 our @ISA = qw(Exporter PortageXS);
 our @EXPORT = qw(
 			getArch
+			getProfileTree
 			getPortageMakeParam
 			getPortageMakeParamHelper
 			getPortdir
@@ -33,6 +45,8 @@ our @EXPORT = qw(
 			getParamFromFile
 			getUseSettingsOfInstalledPackage
 			getAvailableEbuilds
+			getAvailableEbuildVersions
+			getBestEbuildVersion
 			getPortageXScategorylist
 			getAvailableArches
 			getPackagesFromCategory
@@ -63,25 +77,35 @@ sub getArch {
 	return $self->getPortageMakeParam('ARCH');
 }
 
+# Description:
+# Returns the profile tree as array
+# "depth ﬁrst, left to right, with duplicate parent paths being sourced
+# for every time they are encountered"
+sub getProfileTree {
+	my $self	= shift;
+	my $curPath	= shift;
+	my @path;
+
+	if ( -e "$curPath/parent" ) {
+		my $parent=$self->getFileContents("$curPath/parent");
+		foreach (split /\n/, $parent) {
+			push @path, $self->getProfileTree("$curPath/$_");
+		}
+	}
+	push @path, $curPath;
+	return @path;
+}
+
 # Description:
 # Helper for getPortageMakeParam()
 sub getPortageMakeParamHelper {
 	my $self	= shift;
 	my $curPath	= shift;
 	my @files	= ();
-	my $parent	= '';
 
-	if (-e $curPath.'/make.defaults') {
-		push(@files,$curPath.'/make.defaults');
+	foreach my $profile ( $self->getProfileTree($curPath) ) {
+		push(@files,"$profile/make.defaults") if (-e "$profile/make.defaults");
 	}
-	if (! -e $curPath.'/parent') {
-		return @files;
-	}
-	$parent=$self->getFileContents($curPath.'/parent');
-	foreach (split(/\n/,$parent)) {
-		push(@files,$self->getPortageMakeParamHelper($curPath.'/'.$_));
-	}
-
 	return @files;
 }
 
@@ -94,11 +118,11 @@ sub getPortageMakeParam {
 	my $self		= shift;
 	my $param		= shift;
 	my @files		= ();
-	my @etcfiles		= qw(/etc/make.globals /etc/make.conf);
-	my @profilefiles	= ();
+	my @etcfiles		= qw(/usr/share/portage/config/make.globals /etc/make.conf);
 	my $v			= '';
 	my $parent		= '';
-	
+	my $curPath;
+
 	if(!-e $self->{'MAKE_PROFILE_PATH'}) {
 		$self->print_err('Profile not set!');
 		exit(0);
@@ -106,22 +130,21 @@ sub getPortageMakeParam {
 	else {
 		$curPath=$self->getProfilePath();
 	}
-	
-	@profilefiles=$self->getPortageMakeParamHelper($curPath);
-	push(@files,reverse(@profilefiles));
+
+	@files=$self->getPortageMakeParamHelper($curPath);
 	push(@files,@etcfiles);
-	
+
 	foreach (@files) {
 		my $importer = Shell::EnvImporter->new(	shell		=> "bash",
 							file		=> $_,
 							auto_run	=> 1,
 							auto_import	=> 1
 					);
-		
+
 		$importer->shellobj->envcmd('set');
 		$importer->run();
-		
-		if ($ENV{$param} ne '') {
+
+		if ($ENV{$param}) {
 			$v=$ENV{$param};
 			$v=~s/\\t/ /g;
 			$v=~s/\t/ /g;
@@ -133,15 +156,15 @@ sub getPortageMakeParam {
 			$v=~s/\s$//;
 			$v=~s/\s{2,}/ /g;
 		}
-		
+
 		$importer->restore_env();
 	}
-	
+
 	# - Defaults >
 	if ($param eq 'PORTDIR' && !$v) {
 		$v='/usr/portage';
 	}
-	
+
 	return $v;
 }
 
@@ -161,12 +184,12 @@ sub getPortageMakeParam {
 sub getPortdir {
 	my $self	= shift;
 	my $forcereload	= shift;
-	
+
 	if ($self->{'PORTDIR'} && !$forcereload) {
 		return $self->{'PORTDIR'};
 	}
 	else {
-		$self->{'PORTDIR'}=$self->getParamFromFile($self->getFileContents('/etc/make.globals').$self->getFileContents('/etc/make.conf'),'PORTDIR','lastseen');
+		$self->{'PORTDIR'}=$self->getParamFromFile($self->getFileContents('/usr/share/portage/config/make.globals').$self->getFileContents('/etc/portage/make.conf'),'PORTDIR','lastseen');
 		return $self->{'PORTDIR'};
 	}
 }
@@ -182,17 +205,21 @@ sub getPortdir {
 sub getPortdirOverlay {
 	my $self	= shift;
 	my $forcereload	= shift;
-	
-	return split(/ /,$self->getParamFromFile($self->getFileContents('/etc/make.globals').$self->getFileContents('/etc/make.conf'),'PORTDIR_OVERLAY','lastseen'));
+
+	return split(/ /,$self->getParamFromFile($self->getFileContents('/usr/share/portage/config/make.globals').$self->getFileContents('/etc/make.conf'),'PORTDIR_OVERLAY','lastseen'));
 }
 
 # Description:
 # Returnvalue is the content of the given file.
 # $filecontent=$pxs->getFileContents($file);
 sub getFileContents {
-	open(FH,'<'.$_[1]) or die('Cannot open file '.$_[1]);
-	my $content = do{local $/; <FH>};
-	close(FH);
+	my $self = shift;
+	my $file = shift;
+	my $content = do {
+		local $/;
+		open my $fh, '<', $file or die "Cannot open file $file";
+		<$fh>;
+	};
 	return $content;
 }
 
@@ -210,7 +237,7 @@ sub searchInstalledPackage {
 	my $dhp;
 	my $tc;
 	my $tp;
-	
+
 	# - escape special chars >
 	$searchString =~ s/\+/\\\+/g;
 
@@ -221,10 +248,10 @@ sub searchInstalledPackage {
 	else {
 		$s_pak=$searchString;
 	}
-	
+
 	$s_cat=~s/\*//g;
 	$s_pak=~s/\*//g;
-	
+
 	# - read categories >
 	$dhc = new DirHandle($self->{'PKG_DB_DIR'});
 	if (defined $dhc) {
@@ -238,7 +265,7 @@ sub searchInstalledPackage {
 					next;
 				}
 			}
-			
+
 			# - not excluded and $_ is a dir?
 			if (! $self->{'EXCLUDE_DIRS'}{$tc} && -d $self->{'PKG_DB_DIR'}.'/'.$tc) {
 				$dhp = new DirHandle($self->{'PKG_DB_DIR'}.'/'.$tc);
@@ -262,7 +289,7 @@ sub searchInstalledPackage {
 		}
 	}
 	undef $dhc;
-	
+
 	return (sort @matches);
 }
 
@@ -290,15 +317,15 @@ sub searchPackage {
 	my $tc;
 	my $tp;
 	my @matches		= ();
-	
+
 	if (!$mode) { $mode='like'; }
 	$repo=$self->{'PORTDIR'} if (!$repo);
 	if (!-d $repo) { return (); }
-	
+
 	# - escape special chars >
 	if ($mode eq 'like') {
 		$searchString =~ s/\+/\\\+/g;
-		
+
 		# - read categories >
 		$dhc = new DirHandle($repo);
 		if (defined $dhc) {
@@ -346,7 +373,7 @@ sub searchPackage {
 		}
 		undef $dhc;
 	}
-	
+
 	return (sort @matches);
 }
 
@@ -364,23 +391,23 @@ sub getParamFromFile {
 	my $d		= 0;
 	my @lines	= ();
 	my $value	= ''; # value of $param
-	
+
 	# - split file in lines >
 	@lines = split(/\n/,$file);
-	
+
 	for($c=0;$c<=$#lines;$c++) {
 		next if $lines[$c]=~m/^#/;
-		
+
 		# - remove comments >
 		$lines[$c]=~s/#(.*)//g;
-		
+
 		# - remove leading whitespaces and tabs >
-		$lines[$c]=~s/^[ |\t]+//;
-		
+		$lines[$c]=~s/^[ \t]+//;
+
 		if ($lines[$c]=~/^$param="(.*)"/) {
 			# single-line with quotationmarks >
 			$value=$1;
-		
+
 			last if ($mode eq 'firstseen');
 		}
 		elsif ($lines[$c]=~/^$param="(.*)/) {
@@ -398,23 +425,23 @@ sub getParamFromFile {
 					$value.=$lines[$d].' ';
 				}
 			}
-		
+
 			last if ($mode eq 'firstseen');
 		}
 		elsif ($lines[$c]=~/^$param=(.*)/) {
 			# - single-line without quotationmarks >
 			$value=$1;
-			
+
 			last if ($mode eq 'firstseen');
 		}
 	}
-	
+
 	# - clean up value >
-	$value=~s/^[ |\t]+//; # remove leading whitespaces and tabs
-	$value=~s/[ |\t]+$//; # remove trailing whitespaces and tabs
+	$value=~s/^[ \t]+//; # remove leading whitespaces and tabs
+	$value=~s/[ \t]+$//; # remove trailing whitespaces and tabs
 	$value=~s/\t/ /g;     # replace tabs with whitespaces
 	$value=~s/ {2,}/ /g;  # replace 1+ whitespaces with 1 whitespace
-	
+
 	return $value;
 }
 
@@ -429,9 +456,7 @@ sub getUseSettingsOfInstalledPackage {
 	my @package_USE		= ();
 	my @USEs		= ();
 	my $hasuse		= '';
-	my $thisUSE		= '';
-	my $thisIUSE		= '';
-	
+
 	if (-e $self->{'PKG_DB_DIR'}.'/'.$package.'/IUSE') {
 		$tmp_filecontents	= $self->getFileContents($self->{'PKG_DB_DIR'}.'/'.$package.'/IUSE');
 	}
@@ -442,11 +467,11 @@ sub getUseSettingsOfInstalledPackage {
 	}
 	$tmp_filecontents	=~s/\n//g;
 	@package_USE		= split(/ /,$tmp_filecontents);
-	
-	foreach $thisIUSE (@package_IUSE) {
+
+	foreach my $thisIUSE (@package_IUSE) {
 		next if ($thisIUSE eq '');
 		$hasuse = '-';
-		foreach $thisUSE (@package_USE) {
+		foreach my $thisUSE (@package_USE) {
 			if ($thisIUSE eq $thisUSE) {
 				$hasuse='';
 				last;
@@ -454,7 +479,7 @@ sub getUseSettingsOfInstalledPackage {
 		}
 		push(@USEs,$hasuse.$thisIUSE);
 	}
-	
+
 	return @USEs;
 }
 
@@ -465,10 +490,10 @@ sub getAvailableEbuilds {
 	my $catPackage	= shift;
 	my $repo	= shift;
 	my @packagelist	= ();
-	
+
 	$repo=$self->{'PORTDIR'} if (!$repo);
 	if (!-d $repo) { return (); }
-	
+
 	if (-e $repo.'/'.$catPackage) {
 		# - get list of ebuilds >
 		my $dh = new DirHandle($repo.'/'.$catPackage);
@@ -478,10 +503,35 @@ sub getAvailableEbuilds {
 			}
 		}
 	}
-	
+
 	return @packagelist;
 }
 
+# Description:
+# @listOfEbuildVersions=$pxs->getAvailableEbuildVersions(category/packagename,[$repo]);
+sub getAvailableEbuildVersions {
+	my $self	= shift;
+	my $catPackage	= shift;
+	my $repo	= shift;
+	my @packagelist;
+
+	@packagelist = map { $self->getEbuildVersion($_) } $self->getAvailableEbuilds($catPackage,$repo);
+
+	return @packagelist;
+}
+
+# Description:
+# $bestVersion=$pxs->getBestEbuildVersion(category/packagename,[$repo]);
+sub getBestEbuildVersion {
+	my $self	= shift;
+	my $catPackage	= shift;
+	my $repo	= shift;
+
+	my @versions = map { PortageXS::Version->new($_) } $self->getAvailableEbuildVersions($catPackage,$repo);
+	my @best_version = sort { $a <=> $b } (@versions);
+	return $best_version[-1];
+}
+
 # Description:
 # @listOfArches=$pxs->getAvailableArches();
 sub getAvailableArches {
@@ -495,7 +545,7 @@ sub getAvailableArches {
 sub getPortageXScategorylist {
 	my $self	= shift;
 	my $category	= shift;
-	
+
 	return split(/\n/,$self->getFileContents($self->{'PORTAGEXS_ETC_DIR'}.'/categories/'.$category.'.list'));
 }
 
@@ -511,10 +561,10 @@ sub getPackagesFromCategory {
 	my $dhp;
 	my $tp;
 	my @packages	= ();
-	
+
 	return () if !$category;
 	$repo=$self->{'PORTDIR'} if (!$repo);
-	
+
 	if (-d $repo.'/'.$category) {
 		$dhp = new DirHandle($repo.'/'.$category);
 		while (defined($tp = $dhp->read)) {
@@ -542,7 +592,7 @@ sub fileBelongsToPackage {
 	my $dhp;
 	my $tc;
 	my $tp;
-	
+
 	# - read categories >
 	$dhc = new DirHandle($self->{'PKG_DB_DIR'});
 	if (defined $dhc) {
@@ -551,21 +601,21 @@ sub fileBelongsToPackage {
 			if (! $self->{EXCLUDE_DIRS}{$tc} && -d $self->{'PKG_DB_DIR'}.'/'.$tc) {
 				$dhp = new DirHandle($self->{'PKG_DB_DIR'}.'/'.$tc);
 				while (defined($tp = $dhp->read)) {
-					open(FH,'<'.$self->{'PKG_DB_DIR'}.'/'.$tc.'/'.$tp.'/CONTENTS') or next;
-					while (<FH>) {
+					open my $fh, '<', $self->{'PKG_DB_DIR'}.'/'.$tc.'/'.$tp.'/CONTENTS' or next;
+					while (<$fh>) {
 						if ($_=~m/$file/) {
 							push(@matches,$tc.'/'.$tp);
 							last;
 						}
 					}
-					close(FH);
+					close $fh;
 				}
 				undef $dhp;
 			}
 		}
 	}
 	undef $dhc;
-	
+
 	return @matches;
 }
 
@@ -576,7 +626,7 @@ sub getFilesOfInstalledPackage {
 	my $self	= shift;
 	my $package	= shift;
 	my @files	= ();
-	
+
 	# - find installed versions & loop >
 	foreach ($self->searchInstalledPackage($package)) {
 		foreach (split(/\n/,$self->getFileContents($self->{PKG_DB_DIR}.'/'.$_.'/CONTENTS'))) {
@@ -595,7 +645,7 @@ sub getEbuildVersion {
 	my $version	= shift;
 	$version =~ s/\.ebuild$//;
 	$version =~ s/^([a-zA-Z0-9\-_\/\+]*)-([0-9\.]+[a-zA-Z]?)/$2/;
-	
+
 	return $version;
 }
 
@@ -606,9 +656,9 @@ sub getEbuildName {
 	my $self	= shift;
 	my $version	= shift;
 	my $name	= $version;
-	
+
 	$version =~ s/^([a-zA-Z0-9\-_\/\+]*)-([0-9\.]+[a-zA-Z]?)/$2/;
-	
+
 	return substr($name,0,length($name)-length($version)-1);
 }
 
@@ -621,13 +671,13 @@ sub getReponame {
 	my $self	= shift;
 	my $repo	= shift;
 	my $repo_name	= '';
-	
+
 	if (-f $repo.'/profiles/repo_name') {
 		$repo_name = $self->getFileContents($repo.'/profiles/repo_name');
 		chomp($repo_name);
 		return $repo_name;
 	}
-	
+
 	return '';
 }
 
@@ -640,14 +690,14 @@ sub resolveMirror {
 	my $self	= shift;
 	my $mirror	= shift;
 	my $buffer	= $self->getFileContents($self->{PORTDIR}.'/profiles/thirdpartymirrors');
-	
+
 	foreach (split(/\n/,$buffer)) {
 		my @p=split(/\t/,$_);
 		if ($mirror eq $p[0]) {
 			return split(/ /,$p[2]);
 		}
 	}
-	
+
 	return;
 }
 
@@ -659,11 +709,11 @@ sub resolveMirror {
 sub getCategories {
 	my $self	= shift;
 	my $repo	= shift;
-	
+
 	if (-e $repo.'/profiles/categories') {
 		return split(/\n/,$self->getFileContents($repo.'/profiles/categories'));
 	}
-	
+
 	return ();
 }
 
@@ -672,15 +722,15 @@ sub getCategories {
 # $path=$pxs->getProfilePath();
 sub getProfilePath {
 	my $self	= shift;
-	
+
 	if (-e $self->{'ETC_DIR'}.readlink($self->{'MAKE_PROFILE_PATH'})) {
 		return $self->{'ETC_DIR'}.readlink($self->{'MAKE_PROFILE_PATH'});
 	}
 	elsif (-e readlink($self->{'MAKE_PROFILE_PATH'})) {
 		return readlink($self->{'MAKE_PROFILE_PATH'});
 	}
-	
-	return undef;
+
+	return;
 }
 
 # Description:
@@ -688,11 +738,11 @@ sub getProfilePath {
 # @packages=$pxs->getPackagesFromWorld();
 sub getPackagesFromWorld {
 	my $self	= shift;
-	
+
 	if (-e $self->{'PATH_TO_WORLDFILE'}) {
 		return split(/\n/,$self->getFileContents($self->{'PATH_TO_WORLDFILE'}));
 	}
-	
+
 	return ();
 }
 
@@ -703,22 +753,22 @@ sub recordPackageInWorld {
 	my $self	= shift;
 	my $package	= shift;
 	my %world	= ();
-	
+
 	# - get packages already recorded in world >
 	foreach ($self->getPackagesFromWorld()) {
 		$world{$_}=1;
 	}
-	
+
 	# - add $package >
 	$world{$package}=1;
-	
+
 	# - write world file >
-	open(FH,'>'.$self->{'PATH_TO_WORLDFILE'}) or die('Cannot write to world file!');
+	open my $fh, '>', $self->{'PATH_TO_WORLDFILE'} or die('Cannot write to world file!');
 	foreach (keys %world) {
-		print FH $_,"\n";
+		print $fh $_,"\n";
 	}
-	close(FH);
-	
+	close $fh;
+
 	return 1;
 }
 
@@ -729,22 +779,22 @@ sub removePackageFromWorld {
 	my $self	= shift;
 	my $package	= shift;
 	my %world	= ();
-	
+
 	# - get packages already recorded in world >
 	foreach ($self->getPackagesFromWorld()) {
 		$world{$_}=1;
 	}
-	
+
 	# - remove $package >
 	$world{$package}=0;
-	
+
 	# - write world file >
-	open(FH,'>'.$self->{'PATH_TO_WORLDFILE'}) or die('Cannot write to world file!');
+	open my $fh,'>', $self->{'PATH_TO_WORLDFILE'} or die('Cannot write to world file!');
 	foreach (keys %world) {
-		print FH $_,"\n" if ($world{$_});
+		print $fh $_,"\n" if ($world{$_});
 	}
-	close(FH);
-	
+	close $fh;
+
 	return 1;
 }
 
@@ -753,16 +803,16 @@ sub removePackageFromWorld {
 # $pxs->resetCaches();
 sub resetCaches {
 	my $self	= shift;
-	
+
 	# - Core >
 	$self->{'PORTDIR'}=undef;
 	$self->{'PORTDIR'}=$self->getPortdir();
-	
+
 	# - Console >
-	
+
 	# - System - getHomedir >
 	$self->{'CACHE'}{'System'}{'getHomedir'}{'homedir'}=undef;
-	
+
 	# - Useflags - getUsedescs >
 	foreach my $k1 (keys %{$self->{'CACHE'}{'Useflags'}{'getUsedescs'}}) {
 		$self->{'CACHE'}{'Useflags'}{'getUsedescs'}{$k1}{'use.desc'}{'initialized'}=undef;
@@ -772,10 +822,10 @@ sub resetCaches {
 		$self->{'CACHE'}{'Useflags'}{'getUsedescs'}{$k1}{'use.desc'}{'use'}=undef;
 		$self->{'CACHE'}{'Useflags'}{'getUsedescs'}{$k1}{'use.local.desc'}=undef;
 	}
-	
+
 	# - Useflags - getUsemasksFromProfile >
 	$self->{'CACHE'}{'Useflags'}{'getUsemasksFromProfile'}{'useflags'}=undef;
-	
+
 	return 1;
 }
 
@@ -795,11 +845,11 @@ sub searchPackageByMaintainer {
 	my $tp;
 	my @matches		= ();
 	my @fields		= ();
-	
-	if (!$mode) { $mode='like'; }
+
+	#if (!$mode) { $mode='like'; }
 	$repo=$self->{'PORTDIR'} if (!$repo);
 	if (!-d $repo) { return (); }
-	
+
 	# - read categories >
 	foreach ($self->searchPackage('','like',$repo)) {
 		if (-e $repo.'/'.$_.'/metadata.xml') {
@@ -812,7 +862,7 @@ sub searchPackageByMaintainer {
 			}
 		}
 	}
-	
+
 	return (sort @matches);
 }
 
@@ -832,11 +882,11 @@ sub searchPackageByHerd {
 	my $tp;
 	my @matches		= ();
 	my @fields		= ();
-	
-	if (!$mode) { $mode='like'; }
+
+	#if (!$mode) { $mode='like'; }
 	$repo=$self->{'PORTDIR'} if (!$repo);
 	if (!-d $repo) { return (); }
-	
+
 	# - read categories >
 	foreach ($self->searchPackage('','like',$repo)) {
 		if (-e $repo.'/'.$_.'/metadata.xml') {
@@ -846,8 +896,50 @@ sub searchPackageByHerd {
 			}
 		}
 	}
-	
+
 	return (sort @matches);
 }
 
 1;
+
+__END__
+
+=pod
+
+=encoding utf-8
+
+=head1 NAME
+
+PortageXS::Core - Core behaviour role for C<PortageXS>
+
+=head1 VERSION
+
+version 0.2.11
+
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+Christian Hartmann <ian@gentoo.org>
+
+=item *
+
+Torsten Veller <tove@gentoo.org>
+
+=item *
+
+Kent Fredric <kentnl@cpan.org>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2013 by Christian Hartmann.
+
+This is free software, licensed under:
+
+  The GNU General Public License, Version 2, June 1991
+
+=cut
