@@ -1,5 +1,14 @@
-package PortageXS::UI::Spinner;
+use strict;
+use warnings;
 
+package PortageXS::UI::Spinner;
+BEGIN {
+  $PortageXS::UI::Spinner::AUTHORITY = 'cpan:KENTNL';
+}
+{
+  $PortageXS::UI::Spinner::VERSION = '0.2.11';
+}
+# ABSTRACT: Dancing \|/- progress bling for consoles.
 # -----------------------------------------------------------------------------
 #
 # PortageXS::UI::Spinner
@@ -25,10 +34,11 @@ our @EXPORT = qw(
 		);
 
 sub new {
-	my $self	= {};
-	$self->{'spinstate'}=0;
+	my $self	= shift;
+	my $spin = bless {}, $self;
+	$spin->{'spinstate'}=0;
 	$|=1;
-	return bless($self);
+	return $spin;
 }
 
 sub spin {
@@ -44,3 +54,45 @@ sub reset {
 }
 
 1;
+
+__END__
+
+=pod
+
+=encoding utf-8
+
+=head1 NAME
+
+PortageXS::UI::Spinner - Dancing \|/- progress bling for consoles.
+
+=head1 VERSION
+
+version 0.2.11
+
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+Christian Hartmann <ian@gentoo.org>
+
+=item *
+
+Torsten Veller <tove@gentoo.org>
+
+=item *
+
+Kent Fredric <kentnl@cpan.org>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2013 by Christian Hartmann.
+
+This is free software, licensed under:
+
+  The GNU General Public License, Version 2, June 1991
+
+=cut
