@@ -1,5 +1,14 @@
-package PortageXS::Useflags;
+use strict;
+use warnings;
 
+package PortageXS::Useflags;
+BEGIN {
+  $PortageXS::Useflags::AUTHORITY = 'cpan:KENTNL';
+}
+{
+  $PortageXS::Useflags::VERSION = '0.2.11';
+}
+# ABSTRACT: Useflag parsing utilities role for PortageXS
 # -----------------------------------------------------------------------------
 #
 # PortageXS::Useflags
@@ -36,7 +45,7 @@ our @EXPORT = qw(
 # $usedesc=getUsedesc('perl','/usr/portage');
 sub getUsedesc {
 	my $self	= shift;
-	
+
 	return ($self->getUsedescs(@_))[0];
 }
 
@@ -53,7 +62,7 @@ sub getUsedescs {
 	my $package	= shift;
 	my @p		= ();
 	my @descs	= ();
-	
+
 	if (-e $repo.'/profiles/use.desc') {
 		if (!$self->{'CACHE'}{'Useflags'}{'getUsedescs'}{$repo}{'use.desc'}{'initialized'}) {
 			foreach (split(/\n/,$self->getFileContents($repo.'/profiles/use.desc'))) {
@@ -64,18 +73,18 @@ sub getUsedescs {
 			}
 			$self->{'CACHE'}{'Useflags'}{'getUsedescs'}{$repo}{'use.desc'}{'initialized'}=1;
 		}
-		
+
 		if ($self->{'CACHE'}{'Useflags'}{'getUsedescs'}{$repo}{'use.desc'}{'use'}{$use}) {
 			push(@descs,$self->{'CACHE'}{'Useflags'}{'getUsedescs'}{$repo}{'use.desc'}{'use'}{$use});
 		}
 	}
-	
+
 	if ($package) {
 		if (-e $repo.'/profiles/use.local.desc') {
 			if (!$self->{'CACHE'}{'Useflags'}{'getUsedescs'}{$repo}{'use.local.desc'}) {
 				$self->{'CACHE'}{'Useflags'}{'getUsedescs'}{$repo}{'use.local.desc'}=$self->getFileContents($repo.'/profiles/use.local.desc');
 			}
-			
+
 			foreach (split(/\n/,$self->{'CACHE'}{'Useflags'}{'getUsedescs'}{$repo}{'use.local.desc'})) {
 				if ($_) {
 					@p=split(/ - /,$_);
@@ -86,7 +95,7 @@ sub getUsedescs {
 			}
 		}
 	}
-	
+
 	return @descs;
 }
 
@@ -96,19 +105,17 @@ sub getUsedescs {
 sub sortUseflags {
 	my $self	= shift;
 	my @useflags	= @_;
-	my @use1	= (); # +
-	my @use2	= (); # -
-	
-	for (my $x=0;$x<=$#useflags;$x++) {
-		if (substr($useflags[$x],0,1) eq '-') {
-			push(@use2,$useflags[$x]);
+	my (@use1,@use2);
+
+	foreach my $useflag (sort @useflags) {
+		if (substr($useflag,0,1) eq '-') {
+			push @use1, $useflag;
 		}
 		else {
-			push(@use1,$useflags[$x]);
+			push @use2, $useflag;
 		}
 	}
-	
-	return sort(@use1),sort(@use2);
+	return (@use2,@use1);
 }
 
 # Description:
@@ -148,7 +155,7 @@ sub getUsemasksFromProfile {
 	my $c		= 0;
 	my %maskedUses	= ();
 	my @useflags	= ();
-	
+
 	if (!$self->{'CACHE'}{'Useflags'}{'getUsemasksFromProfile'}{'useflags'}) {
 		if(!-e $self->{'MAKE_PROFILE_PATH'}) {
 			$self->print_err('Profile not set!');
@@ -157,7 +164,7 @@ sub getUsemasksFromProfile {
 		else {
 			$curPath=$self->getProfilePath();
 		}
-		
+
 # 		while(1) {
 # 			print "-->".$curPath."<--\n";
 # 			if (-e $curPath.'/use.mask') {
@@ -169,20 +176,20 @@ sub getUsemasksFromProfile {
 # 			$curPath.='/'.$parent;
 # 		}
 		@files = $self->getUsemasksFromProfileHelper($curPath);
-		
+
 		$buffer.=$self->getFileContents($self->{'PORTDIR'}.'/profiles/base/use.mask')."\n";
 		foreach(reverse(@files)) {
 			$buffer.=$self->getFileContents($_)."\n";
 		}
-		
+
 		# - split file in lines >
 		@lines = split(/\n/,$buffer);
-		
+
 		for($c=0;$c<=$#lines;$c++) {
 			next if $lines[$c]=~m/^#/;
 			next if $lines[$c] eq "\n";
 			next if $lines[$c] eq '';
-			
+
 			if (substr($lines[$c],0,1) eq '-') {
 				# - unmask use >
 				$maskedUses{substr($lines[$c],1,length($lines[$c])-1)}=0;
@@ -191,21 +198,63 @@ sub getUsemasksFromProfile {
 				$maskedUses{$lines[$c]}=1;
 			}
 		}
-		
+
 		foreach (keys %maskedUses) {
 			if ($maskedUses{$_}) {
 				push(@useflags,$_);
 			}
 		}
-		
+
 		# - Setup cache >
 		$self->{'CACHE'}{'Useflags'}{'getUsemasksFromProfile'}{'useflags'}=join(' ',@useflags);
 	}
 	else {
 		@useflags=split(/ /,$self->{'CACHE'}{'Useflags'}{'getUsemasksFromProfile'}{'useflags'});
 	}
-	
+
 	return @useflags;
 }
 
 1;
+
+__END__
+
+=pod
+
+=encoding utf-8
+
+=head1 NAME
+
+PortageXS::Useflags - Useflag parsing utilities role for PortageXS
+
+=head1 VERSION
+
+version 0.2.11
+
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+Christian Hartmann <ian@gentoo.org>
+
+=item *
+
+Torsten Veller <tove@gentoo.org>
+
+=item *
+
+Kent Fredric <kentnl@cpan.org>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2013 by Christian Hartmann.
+
+This is free software, licensed under:
+
+  The GNU General Public License, Version 2, June 1991
+
+=cut
